#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-09-26
# File  : inv_cisco_cellular_lte.py
#
# inventory of Cisco cellular interfaces (SIM cards)
#
#

from typing import List
from dataclasses import dataclass
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    TableRow,
    OIDEnd,
    OIDBytes,
    all_of,
    contains,
    exists,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
    InventoryResult,
)


@dataclass
class CiscoCellularProfile:
    apn: str


_cisco_gsm_service_type = {
    '0': '1xRTT',
    '1': 'EVDO Revision 0',
    '2': 'EVDO Revision A',
    '3': 'EVDO Revision B',
    '4': 'GPRS',
    '5': 'EDGE',
    '6': 'UMTS/WCDMA',
    '7': 'HSDPA',
    '8': 'HSUPA',
    '9': 'HSPA',
    '10': 'HSPA Plus',
    '11': 'LTE TDD',
    '12': 'LTE FDD',
}

_cisco_gsm_band = {
    '1': 'unknown',
    '2': 'invalid',
    '3': 'none',
    '4': 'GSM-850',
    '5': 'GSM-900',
    '6': 'GSM-1800',
    '7': 'GSM-1900',
    '8': 'WCDMA-800',
    '9': 'WCDMA-850',
    '10': 'WCDMA-1900',
    '11': 'WCDMA-2100',
    '12': 'LTE Band',
}

_cisco_wan_cell_ext_protocol_type = {
    '1': 'unknown',
    '2': 'IPv4',
    '3': 'PPP',
    '4': 'IPv6',
    '5': 'IPv4v6',
}

# https://en.wikipedia.org/wiki/Mobile_country_code
_mobile_country_code = {
    "202": ("Greece", "GR"),
    "204": ("Netherlands (Kingdom of the Netherlands)", "NL"),
    "206": ("Belgium", "BE"),
    "208": ("France", "FR"),
    "212": ("Monaco", "MC"),
    "213": ("Andorra", "AD"),
    "214": ("Spain", "ES"),
    "216": ("Hungary", "HU"),
    "218": ("Bosnia and Herzegovina", "BA"),
    "219": ("Croatia", "HR"),
    "220": ("Serbia", "RS"),
    "221": ("Kosovo", "XK"),
    "222": ("Italy", "IT"),
    "226": ("Romania", "RO"),
    "228": ("Switzerland", "CH"),
    "230": ("Czech Republic", "CZ"),
    "231": ("Slovakia", "SK"),
    "232": ("Austria", "AT"),
   # "234": ("Guernsey (United Kingdom)", "GG"),
   # "234": ("Jersey (United Kingdom)", "JE"),
   # "234": ("Isle of Man (United Kingdom)", "IM"),
    "234": ("United Kingdom", "GB"),
    "235": ("United Kingdom", "GB"),
    "238": ("Denmark (Kingdom of Denmark)", "DK"),
    "240": ("Sweden", "SE"),
    "242": ("Norway", "NO"),
    "244": ("Finland", "FI"),
    "246": ("Lithuania", "LT"),
    "247": ("Latvia", "LV"),
    "248": ("Estonia", "EE"),
    "250": ("Russian Federation", "RU"),
    "255": ("Ukraine", "UA"),
    "257": ("Belarus", "BY"),
    "259": ("Moldova", "MD"),
    "260": ("Poland", "PL"),
    "262": ("Germany", "DE"),
    "266": ("Gibraltar (United Kingdom)", "GI"),
    "268": ("Portugal", "PT"),
    "270": ("Luxembourg", "LU"),
    "272": ("Ireland", "IE"),
    "274": ("Iceland", "IS"),
    "276": ("Albania", "AL"),
    "278": ("Malta", "MT"),
    "280": ("Cyprus", "CY"),
    "282": ("Georgia", "GE"),
    "283": ("Armenia", "AM"),
    "284": ("Bulgaria", "BG"),
    "286": ("Turkey", "TR"),
    "288": ("Faroe Islands (Kingdom of Denmark)", "FO"),
    "289": ("Abkhazia", "GE-AB"),
    "290": ("Greenland (Kingdom of Denmark)", "GL"),
    "292": ("San Marino", "SM"),
    "293": ("Slovenia", "SI"),
    "294": ("North Macedonia", "MK"),
    "295": ("Liechtenstein", "LI"),
    "297": ("Montenegro", "ME"),
    "302": ("Canada", "CA"),
    "308": ("Saint Pierre and Miquelon", "PM"),
   # "310": ("Guam (United States of America)", "GU"),
   # "310": ("Northern Mariana Islands (United States of America)", "MP"),
     "310": ("United States of America", "US"),
   # "311": ("Guam (United States of America)", "GU"),
    "311": ("United States of America", "US"),
    "312": ("United States of America", "US"),
    "313": ("United States of America", "US"),
    "314": ("United States of America", "US"),
    "315": ("United States of America", "US"),
    "316": ("United States of America", "US"),
    "330": ("Puerto Rico (United States of America)", "PR"),
    "332": ("United States Virgin Islands", "VI"),
    "334": ("Mexico", "MX"),
    "338": ("Jamaica", "JM"),
   # "340": ("Guadeloupe (France)", "GP"),
   # "340": ("Martinique (France)", "MQ"),
   # "340": ("Saint Barthélemy (France)", "BL"),
    "340": ("Saint Martin (France)", "MF"),
    "342": ("Barbados", "BB"),
    "344": ("Antigua and Barbuda", "AG"),
    "346": ("Cayman Islands (United Kingdom)", "KY"),
    "348": ("British Virgin Islands (United Kingdom)", "VG"),
    "350": ("Bermuda", "BM"),
    "352": ("Grenada", "GD"),
    "354": ("Montserrat (United Kingdom)", "MS"),
    "356": ("Saint Kitts and Nevis", "KN"),
    "358": ("Saint Lucia", "LC"),
    "360": ("Saint Vincent and the Grenadines", "VC"),
   # "362": ("Bonaire, Sint Eustatius and Saba", "BQ"),
   # "362": ("Curaçao", "CW"),
    "362": ("Sint Maarten", "SX"),
    "363": ("Aruba", "AW"),
    "364": ("Bahamas", "BS"),
    "365": ("Anguilla (United Kingdom)", "AI"),
    "366": ("Dominica", "DM"),
    "368": ("Cuba", "CU"),
    "370": ("Dominican Republic", "DO"),
    "372": ("Haiti", "HT"),
    "374": ("Trinidad and Tobago", "TT"),
    "376": ("Turks and Caicos Islands", "TC"),
    "400": ("Azerbaijan", "AZ"),
    "401": ("Kazakhstan", "KZ"),
    "402": ("Bhutan", "BT"),
    "404": ("India", "IN"),
    "405": ("India", "IN"),
    "406": ("India", "IN"),
    "410": ("Pakistan", "PK"),
    "412": ("Afghanistan", "AF"),
    "413": ("Sri Lanka", "LK"),
    "414": ("Myanmar", "MM"),
    "415": ("Lebanon", "LB"),
    "416": ("Jordan", "JO"),
    "417": ("Syria", "SY"),
    "418": ("Iraq", "IQ"),
    "419": ("Kuwait", "KW"),
    "420": ("Saudi Arabia", "SA"),
    "421": ("Yemen", "YE"),
    "422": ("Oman", "OM"),
    "424": ("United Arab Emirates", "AE"),
   # "425": ("Israel", "IL"),
    "425": ("Israel/Palestine", "IL/PS"),
    "426": ("Bahrain", "BH"),
    "427": ("Qatar", "QA"),
    "428": ("Mongolia", "MN"),
    "429": ("Nepal", "NP"),
    "430": ("United Arab Emirates (Abu Dhabi)", "AE"),
    "431": ("United Arab Emirates (Dubai)", "AE"),
    "432": ("Iran", "IR"),
    "434": ("Uzbekistan", "UZ"),
    "436": ("Tajikistan", "TJ"),
    "437": ("Kyrgyzstan", "KG"),
    "438": ("Turkmenistan", "TM"),
    "440": ("Japan", "JP"),
    "441": ("Japan", "JP"),
    "450": ("Korea, South", "KR"),
    "452": ("Vietnam", "VN"),
    "454": ("Hong Kong (People’s Republic of China)", "HK"),
    "455": ("Macau (People’s Republic of China)", "MO"),
    "456": ("Cambodia", "KH"),
    "457": ("Laos", "LA"),
    "460": ("China", "CN"),
    "461": ("China", "CN"),
    "466": ("Taiwan", "TW"),
    "467": ("Korea, North", "KP"),
    "470": ("Bangladesh", "BD"),
    "472": ("Maldives", "MV"),
    "502": ("Malaysia", "MY"),
    "505": ("Australia", "AU"),
   # "505": ("Norfolk Island", "NF"),
    "510": ("Indonesia", "ID"),
    "514": ("East Timor", "TL"),
    "515": ("Philippines", "PH"),
    "520": ("Thailand", "TH"),
    "525": ("Singapore", "SG"),
    "528": ("Brunei", "BN"),
    "530": ("New Zealand", "NZ"),
    "536": ("Nauru", "NR"),
    "537": ("Papua New Guinea", "PG"),
    "539": ("Tonga", "TO"),
    "540": ("Solomon Islands", "SB"),
    "541": ("Vanuatu", "VU"),
    "542": ("Fiji", "FJ"),
    "543": ("Wallis and Futuna", "WF"),
    "544": ("American Samoa (United States of America)", "AS"),
    "545": ("Kiribati", "KI"),
    "546": ("New Caledonia", "NC"),
    "547": ("French Polynesia (France)", "PF"),
    "548": ("Cook Islands (Pacific Ocean)", "CK"),
    "549": ("Samoa", "WS"),
    "550": ("Micronesia, Federated States of", "FM"),
    "551": ("Marshall Islands", "MH"),
    "552": ("Palau", "PW"),
    "553": ("Tuvalu", "TV"),
    "554": ("Tokelau", "TK"),
    "555": ("Niue", "NU"),
    "602": ("Egypt", "EG"),
    "603": ("Algeria", "DZ"),
    "604": ("Morocco", "MA"),
    "605": ("Tunisia", "TN"),
    "606": ("Libya", "LY"),
    "607": ("Gambia", "GM"),
    "608": ("Senegal", "SN"),
    "609": ("Mauritania", "MR"),
    "610": ("Mali", "ML"),
    "611": ("Guinea", "GN"),
    "612": ("Ivory Coast", "CI"),
    "613": ("Burkina Faso", "BF"),
    "614": ("Niger", "NE"),
    "615": ("Togo", "TG"),
    "616": ("Benin", "BJ"),
    "617": ("Mauritius", "MU"),
    "618": ("Liberia", "LR"),
    "619": ("Sierra Leone", "SL"),
    "620": ("Ghana", "GH"),
    "621": ("Nigeria", "NG"),
    "622": ("Chad", "TD"),
    "623": ("Central African Republic", "CF"),
    "624": ("Cameroon", "CM"),
    "625": ("Cape Verde", "CV"),
    "626": ("São Tomé and Príncipe", "ST"),
    "627": ("Equatorial Guinea", "GQ"),
    "628": ("Gabon", "GA"),
    "629": ("Congo", "CG"),
    "630": ("Democratic Republic of the Congo", "CD"),
    "631": ("Angola", "AO"),
    "632": ("Guinea-Bissau", "GW"),
    "633": ("Seychelles", "SC"),
    "634": ("Sudan", "SD"),
    "635": ("Rwanda", "RW"),
    "636": ("Ethiopia", "ET"),
    "637": ("Somalia", "SO"),
    "638": ("Djibouti", "DJ"),
    "639": ("Kenya", "KE"),
    "640": ("Tanzania", "TZ"),
    "641": ("Uganda", "UG"),
    "642": ("Burundi", "BI"),
    "643": ("Mozambique", "MZ"),
    "645": ("Zambia", "ZM"),
    "646": ("Madagascar", "MG"),
    "647": ("French Indian Ocean Territories (France)", "RE/YT"),
   # "647": ("French Indian Ocean Territories (France)", "YT"),
    "648": ("Zimbabwe", "ZW"),
    "649": ("Namibia", "NA"),
    "650": ("Malawi", "MW"),
    "651": ("Lesotho", "LS"),
    "652": ("Botswana", "BW"),
    "653": ("Swaziland", "SZ"),
    "654": ("Comoros", "KM"),
    "655": ("South Africa", "ZA"),
    "657": ("Eritrea", "ER"),
    "658": ("Saint Helena, Ascension and Tristan da Cunha", "SH"),
    "659": ("South Sudan", "SS"),
    "702": ("Belize", "BZ"),
    "704": ("Guatemala", "GT"),
    "706": ("El Salvador", "SV"),
    "708": ("Honduras", "HN"),
    "710": ("Nicaragua", "NI"),
    "712": ("Costa Rica", "CR"),
    "714": ("Panama", "PA"),
    "716": ("Peru", "PE"),
    "722": ("Argentina", "AR"),
    "724": ("Brazil", "BR"),
    "730": ("Chile", "CL"),
    "732": ("Colombia", "CO"),
    "734": ("Venezuela", "VE"),
    "736": ("Bolivia", "BO"),
    "738": ("Guyana", "GY"),
    "740": ("Ecuador", "EC"),
    "742": ("French Guiana (France)", "GF"),
    "744": ("Paraguay", "PY"),
    "746": ("Suriname", "SR"),
    "748": ("Uruguay", "UY"),
    "750": ("Falkland Islands (United Kingdom)", "FK"),
    "995": ("British Indian Ocean Territory (United Kingdom)", "IO"),
}


def parse_inv_cisco_cellular_lte(string_table: List[StringByteTable]):
    section = []
    profiles = {}

    modem_table, profile_table = string_table

    for profile_oid_end, apn in profile_table:
        radio_index, profile_index = profile_oid_end.split('.')
        profiles[radio_index] = CiscoCellularProfile(
            apn=apn,
        )

    for oid_end, service_type, imsi, imei, iccid, msisdn, country, network, mcc, current_band in modem_table:
        if not country and mcc:
            country = _mobile_country_code.get(mcc, (None, None))[0]

        entry = {
            'key_columns': {
                'iccid': iccid,
                'imei': imei,
            },
            'inventory_columns': {
                'imsi': imsi if imsi else None,
                'msisdn': msisdn if msisdn else None,
                'country': country if country else None,
                'network': network if network else None,
                'current_band': _cisco_gsm_band.get(current_band, f'Unknown {current_band}'),
                'apn': profiles[oid_end].apn if profiles.get(oid_end) else None,
                'service_type': _cisco_gsm_service_type.get(str(service_type[0] * 256 + service_type[1]))
            },
            'status_columns': {

            }
        }

        keys = list(entry['inventory_columns'].keys())
        for key in keys:
            if not entry['inventory_columns'][key]:
                entry['inventory_columns'].pop(key)

        section.append(entry)

    return section


def inventory_cisco_cellular_lte(params, section) -> InventoryResult:
    path = ['networking', 'cellular']

    for entry in section:
        yield TableRow(
            path=path,
            key_columns=entry['key_columns'],
            inventory_columns=entry['inventory_columns'],
            status_columns=entry['status_columns']
        )


register.snmp_section(
    name='inv_cisco_cellular_lte',
    parse_function=parse_inv_cisco_cellular_lte,
    fetch=[SNMPTree(
        base='.1.3.6.1.4.1.9.9.661.1',  # CISCO-WAN-3G-MIB::ciscoWan3gMIBObjects
        oids=[
            OIDEnd(),
            OIDBytes('1.1.5'),  # c3gCurrentServiceType
            '3.1.1.1',  # c3gImsi
            '3.1.1.2',  # c3gImei
            '3.1.1.3',  # c3gIccId
            '3.1.1.4',  # c3gMsisdn
            '3.2.1.8',  # c3gGsmCountry
            '3.2.1.9',  # c3gGsmNetwork
            '3.2.1.10',  # c3gGsmMcc
            '3.4.1.1.3',  # c3gGsmCurrentBand
        ]),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.817.1.1.2.3.1',  # CISCO-WAN-CELL-EXT-MIB::cwceLteProfileEntry
            oids=[
                OIDEnd(),
                '5',  # cwceLteProfileApn
            ])],
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),  # sysDescr
        exists('.1.3.6.1.4.1.9.9.661.1.3.1.1.2.*'),  # CISCO-WAN-3G-MIB::c3gImei
    ))

register.inventory_plugin(
    name='inv_cisco_cellular_lte',
    inventory_function=inventory_cisco_cellular_lte,
    inventory_ruleset_name='inv_cisco_cellular_lte',
    inventory_default_parameters={},
)
